#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include "libtsctl.h"


int Lookup(System *sys,char *str,...) {
  va_list ap;
  va_start(ap,str);
  int len,n;
  char dummy[1];

  len = vsnprintf(dummy,0,str,ap); 
  {
    char name[len];
    vsprintf(name,str,ap);  
    n = sys->API->MapLookup(sys,name);
  }
  va_end(ap);
  return n;
}
 
void error(int output,int input,int driven,int read) {
  printf("DIO %d driven %s, DIO %d read back %s\n",
	 output,DIOValueString(driven),input,DIOValueString(read));
}

// TO DO: sync versus async
int main(int argc,char *argv[]) {
  System *sys = SystemInit(0);
  DIO *dio = DIOInit(0);
  unsigned model,bb;
  char *cf,configfile[256+64],name[256],*str,*str2;
  int i,i2,j,maxconn,wires,*dionum,*dioval,*diocap,cn,pin,val,tested[256];
  int passed=0,failed=0;

  // get model id and verify this is a board we can test
  model = sys->API->ModelId(sys);
  printf("Model=%04X\n",model);
  if (model < 0 || (model & 0xF000) != 0x4000) {
    fprintf(stderr,"TS-4XXX socket module required\n");
    return 1;
  }
  // get baseboard id
  bb = sys->API->BaseBoardId(sys);
  printf("BaseBoard=%04X\n",bb);
  if (argc == 2) {
    cf = argv[1];
    printf("Using config file %s\n",cf);
  } else {
    sprintf(configfile,"./ts%04X.dioctl.config",bb);
    cf = configfile;
  }
  if (!DIOMapLoadFromFile(cf)) {
    sprintf(configfile,"/etc/ts%04X.dioctl.config",bb);
    if (!DIOMapLoadFromFile(configfile)) {
      fprintf(stderr,"Can't find %s in . or /etc\n",cf,bb);
      return 1;
    }
  }
  // make sure we have a DIO object
  if (!dio) {
    fprintf(stderr,"No DIO 0 object\n");
    return 1;
  }
  if (dio->InitStatus <= 0) {
    fprintf(stderr,"Error %d initializing DIO 0\n",-dio->InitStatus);
    return 1;
  }
  // look up how the DIOs are connected on the base board
  maxconn = Lookup(sys,"attrib.%X.Wire.MaxConnections",bb);
  printf("Max connections: %d\n",maxconn);
  wires = Lookup(sys,"attrib.%X.Wire.Count",bb);
  printf("Wires: %d\n",wires);
  dionum = malloc(sizeof(int) * wires * maxconn);
  dioval = malloc(sizeof(int) * wires * maxconn);
  diocap = malloc(sizeof(int) * wires * maxconn);
  for (i=0;i<wires;i++) {
    for (j=0;j<maxconn;j++) {
      dionum[j+i*maxconn] = -1;
      cn = Lookup(sys,"attrib.%X.Wire.Connector.%d.%d",bb,i,j);
      if (cn >= 0) {
	pin = Lookup(sys,"attrib.%X.Wire.Pin.%d.%d",bb,i,j);
	if (sys->API->MapLookupPartial(sys,
				       "attrib.Connector.Name.",cn,name)>0) {
	  dionum[j+i*maxconn] = Lookup(sys,"%s%d",name,pin);
	  diocap[j+i*maxconn] = dio->API->Capabilities(dio,dionum[j+i*maxconn]);
	  //printf("#%d=%d [%d]\n",j+i*maxconn,dionum[j+i*maxconn],diocap[j+i*maxconn]);
	  //printf("wire %d, conn %d = %s%d = %d\n",
	  //       i,j,name,pin,dionum[j+i*maxconn]);
	}
      }      
    }
  }
  // lock all the DIOs to be tested
  for (i=0;i<wires*maxconn;i++) {
    if (dionum[i] >= 0) {
      dio->API->Lock(dio,dionum[i],0);
    }
  }
  // get current state of all DIOs
  for (i=0;i<wires*maxconn;i++) {
    if (dionum[i] >= 0) {
      dioval[i] = dio->API->GetAsync(dio,dionum[i]);
    }
  }
  // set all DIOs to inputs
  for (i=0;i<wires*maxconn;i++) {
    if (dionum[i] >= 0) {
      dio->API->SetAsync(dio,dionum[i],INPUT);
    }
  }
  // run the test in asynchronous mode
  for (j=0;j<wires;j++) {
    for (i=0;i<maxconn;i++) {
      if (dionum[i+j*maxconn] == -1) continue;
      if (diocap[i+j*maxconn] & DIO_CAN_DRIVE_LOW) {
	// set DIO on wire i to output low
	dio->API->SetAsync(dio,dionum[i+j*maxconn],LOW);
	// check that other wires are low
	for (i2=0;i2<maxconn;i2++) {
	  if (dionum[i2+j*maxconn] >= 0
	      && (diocap[i2+j*maxconn] & DIO_CAN_INPUT)) {
	    val = dio->API->GetAsync(dio,dionum[i2+j*maxconn]);
	    if (val != ((i2==i)?LOW:INPUT_LOW)) {
	      error(dionum[i+j*maxconn],dionum[i2+j*maxconn],0,val);
	      failed++;
	    } else passed++;
	  }
	}
      }
      if (diocap[i+j*maxconn] & DIO_CAN_DRIVE_HIGH) {
	// set DIO on wire i to output high
	dio->API->SetAsync(dio,dionum[i+j*maxconn],HIGH);
	// check that other wires are high
	for (i2=0;i2<maxconn;i2++) {
	  if (dionum[i2+j*maxconn] >= 0
	      && (diocap[i2+j*maxconn] & DIO_CAN_INPUT)) {
	    val = dio->API->GetAsync(dio,dionum[i2+j*maxconn]);
	    if (val != ((i2==i)?HIGH:INPUT_HIGH)) {
	      error(dionum[i+j*maxconn],dionum[i2+j*maxconn],1,val);
	      failed++;
	    } else passed++;
	  }
	}
      }
      // set DIO on wire i to back to input
      dio->API->SetAsync(dio,dionum[i+j*maxconn],INPUT);
    }
  }
  // restore state of all DIOs
  for (i=0;i<wires*maxconn;i++) {
    if (dionum[i] >= 0) {
      dio->API->SetAsync(dio,dionum[i],dioval[i]);
    }
  }
  // unlock all the DIOs
  for (i=0;i<wires*maxconn;i++) {
    if (dionum[i] >= 0) {
      dio->API->Unlock(dio,dionum[i],0);
    }
  }
  printf("%d passed, %d failed\n",passed,failed);
  exit(0);
}
